@@ -93,6 +93,8 @@ static inline int ThreadSetName(const char* name) {
 #elif defined(__HAIKU__)
 	rename_thread(find_thread(NULL), name);
 	return 0;
+#elif defined(__NetBSD__)
+	return pthread_setname_np(pthread_self(), "%s", (void *) name);
 #elif defined(HAVE_PTHREAD_SETNAME_NP)
 	return pthread_setname_np(pthread_self(), name);
 #else
@@ -101,6 +103,22 @@ static inline int ThreadSetName(const char* name) {
 #endif
 }
 
+#if (__STDC_VERSION__ < 201112L) || (__STDC_NO_THREADS__ == 1)
+typedef pthread_key_t ThreadLocal;
+
+static inline void ThreadLocalInitKey(ThreadLocal* key) {
+	pthread_key_create(key, 0);
+}
+
+static inline void ThreadLocalSetKey(ThreadLocal key, void* value) {
+	pthread_setspecific(key, value);
+}
+
+static inline void* ThreadLocalGetValue(ThreadLocal key) {
+	return pthread_getspecific(key);
+}
+#endif
+
 CXX_GUARD_END
 
 #endif
