@@ -79,6 +79,20 @@ void ActionMapper::rebuildMenu(const QString& menu, QMenu* qmenu, QWidget* conte
 		} else if (!m_defaultShortcuts[actionName].isEmpty()) {
 			qaction->setShortcut(m_defaultShortcuts[actionName][0]);
 		}
+		switch (action->role()) {
+		case Action::Role::NO_ROLE:
+			qaction->setMenuRole(QAction::NoRole);
+			break;
+		case Action::Role::SETTINGS:
+			qaction->setMenuRole(QAction::PreferencesRole);
+			break;
+		case Action::Role::ABOUT:
+			qaction->setMenuRole(QAction::AboutRole);
+			break;
+		case Action::Role::QUIT:
+			qaction->setMenuRole(QAction::QuitRole);
+			break;
+		}
 		QObject::connect(qaction, &QAction::triggered, [qaction, action](bool enabled) {
 			if (qaction->isCheckable()) {
 				action->trigger(enabled);
@@ -172,4 +186,4 @@ Action* ActionMapper::getAction(const QString& itemName) {
 
 QKeySequence ActionMapper::defaultShortcut(const QString& itemName) {
 	return m_defaultShortcuts[itemName];
-}
\ No newline at end of file
+}
